#!/usr/bin/env python3

"""
abs1.py
Prepare a suitable synthetic spectrum from BSTAR grid.

"""

__author__ = "Miroslav Broz (miroslav.broz@email.cz)"
__version__ = "Jul 24th 2017"

import sys
import string
import numpy as np

from scipy.interpolate import splrep, splev
from pyterpol3.synthetic.auxiliary import instrumental_broadening

cm = 1.e-2		# m
nm = 1.e-9		# m
Ang = 1.e-10		# m
erg = 1.e-7		# J
c = 299792458.		# m s^-1
h = 6.63e-34		# J s
k = 1.38e-23		# J K^-1

def B(lambda_,T):
    return 2.*h*c**2/lambda_**5 * 1 / (np.exp(h*c/(k*T*lambda_)) - 1)

def main():
    
    rule = str.maketrans('D', 'E')
#   converters = {0: lambda val: float(val.translate(rule)), 1: lambda val: float(val.translate(rule))}

    lambda1, H_lambda1 = np.loadtxt('BG30000g400v2.uv.7', dtype=float, unpack=True, usecols=(0,1))
    lambda2, H_lambda2 = np.loadtxt('BG30000g400v2.vis.7', dtype=float, unpack=True, usecols=(0,1))

    # black-body extrapolation
    wmin = 10000.
    wmax = 48000.
    step = 0.1
    lambda3 = np.arange(wmin, wmax, step)

    factor = 0.775
    T = 30000.  # K
    H_lambda3 = np.pi * B(lambda3*Ang, T) / (erg*cm**(-2)*Ang**(-1)) / (4.*np.pi)*factor

    lambda_ = np.concatenate((lambda1, lambda2))
    lambda_ = np.concatenate((lambda_, lambda3))
    H_lambda = np.concatenate((H_lambda1, H_lambda2))
    H_lambda = np.concatenate((H_lambda, H_lambda3))

#    F_lambda = 4.*np.pi*H_lambda

    # Ang -> Ang
    # erg s^-1 cm^-2 A^-1 -> erg s^-1 cm^-2 A^-1 (as in POLLUX)
    wave = lambda_
    intens = H_lambda

    # check delta_lambda = 0.0
    wave, idx = np.unique(wave, return_index=True)
    intens = intens[idx]

    # spline interpolation (to 0.1 A)
    step = 0.1
    wmin = 900.
    wnew = np.arange(wmin, wmax, step)
    
    tck = splrep(wave, intens)
    intens = splev(wnew, tck)
    wave = wnew

    # convolution (to 1 Angstrom)
#    step = 0.1
#    intens = instrumental_broadening(wave, intens, width=step)

    # spline interpolation
    wnew = np.arange(wmin, wmax, step)
    tck = splrep(wave, intens)
    intens = splev(wnew, tck)

    # elliminate negatives!
    for i in range(0,len(intens)):
        if intens[i] < 0.0:
            intens[i] = 0.0

    np.savetxt('starspec1.abs', np.column_stack([wnew, intens]), fmt="%.6e %.10e")

if __name__ == "__main__":
    main()


